@@ -1,9 +1,9 @@
 /*
  * Digium, Inc.  Wildcard TE110P T1/PRI card Driver
  *
- * Written by Mark Spencer <markster@linux-support.net>
- *            Matthew Fredrickson <creslin@linux-support.net>
- *            William Meadows <wmeadows@linux-support.net>
+ * Written by Mark Spencer <markster@digium.com>
+ *            Matthew Fredrickson <creslin@digium.com>
+ *            William Meadows <wmeadows@digium.com>
  *
  * Copyright (C) 2004, Digium, Inc.
  *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
  *
- * $Id: wcte11xp.c,v 1.1.2.5 2005/03/01 14:35:16 russell Exp $
+ * $Id: wcte11xp.c,v 1.1.2.8 2005/04/11 06:15:59 russell Exp $
  */
 
 #include <linux/kernel.h>
@@ -226,6 +226,8 @@ static int t1xxp_open(struct zt_chan *chan)
 	wc->usecount++;
 #ifndef LINUX26	
 	MOD_INC_USE_COUNT;
+#else
+	try_module_get(THIS_MODULE);
 #endif	
 	return 0;
 }
@@ -346,6 +348,8 @@ static int t1xxp_close(struct zt_chan *chan)
 	wc->usecount--;
 #ifndef LINUX26	
 	MOD_DEC_USE_COUNT;
+#else
+	module_put(THIS_MODULE);
 #endif
 	/* If we're dead, release us now */
 	if (!wc->usecount && wc->dead) 
@@ -1463,8 +1467,8 @@ static int __devinit t1xxp_init_one(struct pci_dev *pdev, const struct pci_devic
 			/* Keep track of which device we are */
 			pci_set_drvdata(pdev, wc);
 
-			if (request_irq(pdev->irq, t1xxp_interrupt, SA_INTERRUPT | SA_SHIRQ, "t1xxp", wc)) {
-				printk("t1xxp: Unable to request IRQ %d\n", pdev->irq);
+			if (request_irq(pdev->irq, t1xxp_interrupt, SA_INTERRUPT | SA_SHIRQ, "wcte11xp", wc)) {
+				printk("wcte11xp: Unable to request IRQ %d\n", pdev->irq);
 				kfree(wc);
 				return -EIO;
 			}
@@ -1534,7 +1538,7 @@ static struct pci_device_id t1xxp_pci_tbl[] = {
 MODULE_DEVICE_TABLE(pci,t1xxp_pci_tbl);
 
 static struct pci_driver t1xxp_driver = {
-	name: 	"t1xxp",
+	name: 	"wcte11xp",
 	probe: 	t1xxp_init_one,
 #ifdef LINUX26
 	remove:	__devexit_p(t1xxp_remove_one),
@@ -1573,8 +1577,8 @@ MODULE_PARM(t1e1override, "i");
 MODULE_PARM(clockextra, "i");
 MODULE_PARM(debug, "i");
 #endif
-MODULE_DESCRIPTION("Wildcard T100P/E100P Zaptel Driver");
-MODULE_AUTHOR("Mark Spencer <markster@linux-support.net>");
+MODULE_DESCRIPTION("Wildcard TE110P Zaptel Driver");
+MODULE_AUTHOR("Mark Spencer <markster@digium.com>");
 #ifdef MODULE_LICENSE
 MODULE_LICENSE("GPL");
 #endif
