@@ -267,7 +267,7 @@ int spanconfig(char *keyword, char *args)
 			return -1;
 		}
 	}
-	lc[spans].span = spans + 1;
+	lc[spans].span = span;
 	lc[spans].sync = timing;
 	/* Valid span */
 	spans++;
@@ -676,7 +676,7 @@ static void usage(char *argv0, int exitcode)
 
 int main(int argc, char *argv[])
 {
-	char c;
+	int c;
 	char *buf;
 	char *key, *value;
 	int x,found;
