@@ -1,9 +1,14 @@
 #include <sys/ioctl.h>
-#include <linux/zaptel.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 
+#ifdef STANDALONE_ZAPATA
+#include "zaptel.h"
+#else
+#include <linux/zaptel.h>
+#endif
+
 int main(int argc, char *argv[])
 {
 	int fd;
